import * as quill from 'quill';
export as namespace marcura;
import './core/core.module';
// import {IModalInstanceService, IModalSettings} from 'angular-ui-bootstrap'; // importing types in that way causes problems with building PLO
export = marcura

declare global {
    const marcura: marcura.IMarcuraStatic;
}

declare namespace marcura {
    const module: angular.IModule;
    //Types
    type IUnsubscribe = () => void;
    type IObserverNext<T = any> = (value: T) => void;

    //Global
    interface IMarcuraStatic {
        Observer: { new <T = any>(next: IObserverNext<T>): IObserver<T> };
        Subject: { new <T = any>(): ISubject<T> };
        BehaviourSubject: { new <T = any>(value?: T): IBehaviourSubject<T> };
    }

    //Global
    interface IObserver<T = any> {
        closed?: boolean;
        next(value: T): void;
    }

    interface ISubject<T = any> {
        observers: IObserver<T>[];
        subscribe(observer: IObserver<T>): IUnsubscribe;
        unsubscribe(observer: IObserver<T>): void;
        next(value: T): void;
        dispose(): void;
    }

    interface IBehaviourSubject<T = any> extends ISubject<T> {
        value: T;
        getValue(): T;
    }

    //Services

    interface IApiService {
        getConfig(): angular.IRequestShortcutConfig;
        getUrl(): string;
    }

    interface IApiConfigService {
        getBaseUrl(): string;
        getCharteringBaseUrl(): string;
        hasFeature(feature: string): boolean;
    }

    interface IPortApiService extends IApiService {
        getPorts(): angular.IPromise<Array<Port>>
        getPortById(id: number): angular.IPromise<Port>;
    }

    interface ITerminalApiService extends IApiService {
        getTerminals(portId: number): angular.IPromise<Array<Terminal>>;
        getTerminalsMultiplePorts(portIds: number[]): angular.IPromise<Array<Terminal>>;
        getTerminalById(id: number): angular.IPromise<Terminal>;
    }

    interface ICountryApiService extends IApiService {
        getCountries(): angular.IPromise<Array<Country>>;
        getCountryById(id: number): angular.IPromise<Country>;
    }

    interface IVesselTypeApiService extends IApiService {
        getVesselTypes(): angular.IPromise<Array<VesselType>>;
    }

    interface IVesselApiService extends IApiService {
        getVessels(): angular.IPromise<Array<Vessel>>;
        getVesselById(id: number): angular.IPromise<Vessel>;
    }

    interface ICommodityApiService extends IApiService {
        getCommoditiesData(): angular.IPromise<Array<Commodity>>;
        getCommodities(): angular.IPromise<Array<Commodity>>;
        getSpecificCommodities(): angular.IPromise<Array<Commodity>>;
        getCommodityById(id: number): angular.IPromise<Commodity>;
        getSpecificCommodityById(id: number): angular.IPromise<Commodity>;
        removeMask(id: any): any;
        addMask(id: number, isSpecific: boolean): number;
    }

    interface IInstructionApiService {
        archiveInstruction(instruction: IInstruction): angular.IPromise<IInstruction>;
        canPublish(): boolean;
        canDownload(): boolean;
        deleteInstruction(instruction: IInstruction): angular.IPromise<any>;
        downloadAttachment(attachmentId: string): void
        downloadInstructions(filterConfig: IInstructionFilterConfig): angular.IPromise<void>;
        getCategories(): angular.IPromise<Category[]>;
        getAvailableFilteredInstructions(filterConfig: IInstructionFilterConfig): angular.IPromise<IApiResponse<IInstructionGroup>>
        getFilteredInstructions(filterConfig?: IInstructionFilterConfig): angular.IPromise<IApiResponse<IInstructionGroup>>;
        getInstructions(): angular.IPromise<IApiResponse<IInstructionGroup>>;
        isAdmin(): boolean;
        publishInstruction(instruction: IInstruction): angular.IPromise<any>;
        saveInstruction(tags: any, instruction: InstructionCreatorDescription): angular.IPromise<IInstruction>;
        toggleAttachInstruction(instruction: IInstruction): angular.IPromise<IInstruction>;
        unarchiveInstruction(instruction: IInstruction): angular.IPromise<IInstruction>;
        updateInstruction(instruction: IInstruction): angular.IPromise<IInstruction>;
    }

    interface IInstructionBaseApiService extends IApiService {
        archiveInstruction(instruction: IInstruction): angular.IPromise<IInstruction>;
        canPublish(): boolean;
        canDownload(): boolean;
        deleteInstruction(instruction: IInstruction): angular.IPromise<any>;
        downloadAttachment(attachmentId: string): void;
        getInstructions(): angular.IPromise<IApiResponse<IInstructionGroup>>;
        isAdmin(): boolean;
        publishInstruction(instruction: IInstruction): angular.IPromise<any>;
        saveInstruction(tags: any, instruction: InstructionCreatorDescription): angular.IPromise<marcura.IInstruction>;
        toggleAttachInstruction(instruction: IInstruction): angular.IPromise<IInstruction>;
        unarchiveInstruction(instruction: IInstruction): angular.IPromise<IInstruction>;
        updateInstruction(instruction: IInstruction): angular.IPromise<IInstruction>;
    }

    interface IInstructionFilterApiService extends IApiService {
        downloadInstructions(filterConfig: IInstructionFilterConfig): angular.IPromise<void>;
        getAvailableInstructions(filterConfig?: IInstructionFilterConfig): angular.IPromise<IApiResponse<IInstructionGroup>>;
        getInstructions(filterConfig?: IInstructionFilterConfig): angular.IPromise<IApiResponse<IInstructionGroup>>;
        getFilterConfig(): IInstructionFilterConfig;
    }

    interface IInstructionCategoryApiService extends IApiService {
        getCategories(): angular.IPromise<Category[]>;
    }

    interface IInstructionGroupsService {
    }

    interface IInstructionGroupService {
    }

    interface IInstructionFilterAssigneeProvider extends angular.IServiceProvider {
        assignees: Assignee[];
        $get: () => Assignee[];
        addAssignee(assignee: Assignee): void;
        setAssignees(assignees: Assignee[]): void;
        removeAssigneeByName(name: string): void;
        removeAssigneeById(id: number): void;
    }

    interface IInstructionConfigProvider extends angular.IServiceProvider {
        config: IInstructionConfig;
        $get: () => IInstructionConfig;
    }

    interface IInstructionListService {
        getInstructionsWithCounts(filterConfig: marcura.IInstructionFilterConfig): angular.IPromise<marcura.IInstructions>;
        downloadInstructions(filterConfig: marcura.IInstructionFilterConfig): angular.IPromise<void>;
        getPortcallInstructions(filterConfig?: IInstructionFilterConfig): angular.IPromise<IInstructionsListObject>;
        getInstructions(filterConfig?: IInstructionFilterConfig): angular.IPromise<IInstructionGroup[]>;
        getInstructionsCounts(filterConfig?: IInstructionFilterConfig): angular.IPromise<marcura.IInstructionsManagerCounters>
        applyAssigneesFilter(assignees: Assignee[], groups: IInstructionGroup[], filters: marcura.IInstructionFilter): IInstructionGroup[];
        applyPaging(perPage: number, pageNumber: number, instructions: IInstruction[]): IInstruction[];
    }

    interface IInstructionAssigneesProvider {
        getAssignees(): Assignee[];
    }

    interface IInstructionFilterAssigneeService {
        applyAssigneeFilter(assignees: Assignee[], groups: IInstructionGroup[], filters: IInstructionFilter): IInstructionGroup[];
        countAssignee(assignees: Assignee[], groups: IInstructionGroup[]): Assignee[];
        getActiveFilters(assignees?: Assignee[]): Assignee[];
        getDefaultAssignees(): Assignee[];
        setAssigneesCounts(assignees: Assignee[], counters: ICountedNames<number>): Assignee[];
    }

    interface IInstructionPaginationService {
        hasNext(instructions: IInstruction[], perPage: number, pageNumber: number): boolean;
        getNextPage(instructions: IInstruction[], perPage: number, pageNumber: number): IInstruction[]
    }

    interface IInstructionItemService {
        archive(instrution: IInstruction): angular.IPromise<IInstruction>;
        archive(instrution: IInstruction): angular.IPromise<IInstruction>;
        delete(instrution: IInstruction): angular.IPromise<void>;
        download(attachmentId: string): void;
        getAssigneeTags(instruction: IInstruction, filters?: IInstructionFilter): ITag[];
        getCategoryTags(instruction: IInstruction, filters?: IInstructionFilter): ITag[];
        getCounterparts(instruction: IInstruction, filters: IInstructionFilter): ITag[];
        getRestrictionTags(instruction: IInstruction, filters?: IInstructionFilter): ITag[];
        publish(instruction: IInstruction): angular.IPromise<void>;
        toggleAttach(instruction: IInstruction): angular.IPromise<IInstruction>;
        truncateFileName(fileName: string): string;
        unarchive(instrution: IInstruction): angular.IPromise<IInstruction>;
    }

    interface IInstructionActionService {
        archive(instruction: IInstruction, ...args: any[]): angular.IPromise<IInstruction>;
        copy(instruction: IInstruction, ...args: any[]): void;
        delete(instruction: IInstruction, ...args: any[]): angular.IPromise<void>;
        edit(instruction: IInstruction, ...args: any[]): void;
        publish(instruction: IInstruction, ...args: any[]): angular.IPromise<IInstruction>;
        unarchive(instruction: IInstruction, ...args: any[]): angular.IPromise<IInstruction>;
    }

    interface IFileTypeService {
        getType(file: File): AttachmentFileType | null;
    }

    interface IHttpMapperService {
        objectToURLParams(obj: { [propertyKey: string]: any }): string
    }

    interface IModalWindowService {
        readonly SETTINGS: angular.ui.bootstrap.IModalSettings;
        show: <TReturnType>(settings: angular.ui.bootstrap.IModalSettings) => angular.ui.bootstrap.IModalInstanceService;
        close: (modelInstance: angular.ui.bootstrap.IModalInstanceService, result?: any) => void;
        dismiss: (moduleInstance: angular.ui.bootstrap.IModalInstanceService, reason?: any) => void;
        confirmation: <TReturnType>(
            message: string,
            title?: string,
            confirmCallback?: () => void,
            closeCallback?: () => void,
        ) => angular.ui.bootstrap.IModalInstanceService;
        error: <TReturnType>(
            message: string,
            title?: string,
            callback?: () => void,
        ) => angular.ui.bootstrap.IModalInstanceService;
        success: <TReturnType>(
            message: string,
            title?: string,
            callback?: () => void,
        ) => angular.ui.bootstrap.IModalInstanceService;
    }

    interface IUUIDService {
        getUUID(): UUID;
    }

    interface IUrlService {
        normalize(url: string): string;
    }

    interface IUserService {
        getCurrentUser(): IUser;
    }

    type TInsightPreviousValue =  {name: string, params: object};

    interface IInsightService {
        setPreviousStateValue(state: string, params: object): void;
        getPreviousStateValue(): TInsightPreviousValue,
        clearPreviouStateValue(): void,
    }

    interface ICounterpartApiService extends IApiService {
        counterparts$: marcura.IBehaviourSubject<marcura.ICounterpart[]>;
        fetchCounterparts(): marcura.IBehaviourSubject<marcura.ICounterpart[]>;
        fetchCounterpartsAsPromise(): angular.IPromise<marcura.ICounterpart[]>
    }

    interface ICounterpartService {
        searchQuery(selected: ICounterpart[], query: string): ICounterpart[];
    }

    interface IDraftsApiService extends IApiService {
        getLast5Drafts(stateParams: object): angular.IPromise<marcura.IDraft[]>;
        getMax5Drafts(stateParams: object): angular.IPromise<marcura.IDraft[]>;
    }

    interface IRestrictionsInfoApiService extends IApiService {
        getRestrictionsInfo(stateParams: object): angular.IPromise<marcura.IRestrictionsInfo>;
        getAgentDetails(agentId: number, portId: number): angular.IPromise<marcura.IAgent>;
        getTides(stateParams: object): angular.IPromise<ITides>;
    }

    interface ICommentsApiService extends IApiService {
        getComments(itemObject: any, type: string): any;
        addComment(itemObject: any, comment: string, type: string): any;
        updateComment(commentId: number, comment: string, type: string): any;
        deleteComment(commentId: number, type: string): any;
    }

    interface ILikesApiService extends IApiService {
        toggleLike(itemObject: any): any;
        getLikers(itemObject: any): any;
    }

    interface IUserSettingsApiService extends IApiService {
        getUserFromRootScope(): IUser;
        updateUser(userDetails: IUser): angular.IPromise<angular.IHttpResponse<IUser>>;
        uploadProfilePicture(croppedImage: string): angular.IPromise<angular.IHttpResponse<IUser>>;
        getProfilePicSrc(userId: string, scale: string): string;
        deleteProfilePic(): angular.IPromise<angular.IHttpResponse<IUser>>;
    }

    interface IUserDetailsApiService extends IApiService {
        getCurrentUserDetails(profile?: boolean): angular.IPromise<IUser>;
        isEntitled(userDetails: IUser, entitlement: USER_ENTITLEMENTS): boolean;
        isDry(userDetails: IUser): boolean;
        isWet(userDetails: IUser): boolean;
        isPTLManager(userDetails: IUser): boolean;
    }

    interface ITurnaroundTableApiService extends IApiService {
        turnaroundTable(stateParams: any, getPrevious: boolean, extraParams?: any): any;
        secondsToDaysAndHours(seconds: number, format?: boolean): string;
    }

    interface IHelperService {
        minutesToDaysAndHours(minutes: number, format: boolean): string;
        secondsToDaysAndHours(seconds: number, format: boolean): string;
        formatTimestamp(text: string): string;
        displayGlobalAlert(type: string, message?: string, messageObject?: any): void;
        displayGlobalLoader(): void;
        removeGlobalLoader(): void;
    }

    // Enums - the enums here can be used in other projects normally, but they are duplicated in their own files as well to be used in common :(

    enum FilesSignatureName {
        DOC = 'doc',
        DOCX = 'docx',
        EML = 'eml',
        JEPG = 'jpeg',
        PDF = 'pdf',
        PNG = 'png',
        PPT = 'ppt',
        PPTX = 'pptx',
        XLS = 'xls',
        XLSX = 'xlsx'
    }

    enum InstructionCreatorEventType {
        PREV_STATE = 'prev_state',
        MANAGER_STATE = 'manager_state'
    }

    enum TagsColor {
        DEFAULT = 'default',
        ORANGE = 'orange',
        BLUE = 'blue',
        GREEN = 'green'
    }

    enum PLFeatures {
        ADMINISTRATION = 'ADMINISTRATION',
        CLIENT_ADMINISTRATION = 'CLIENT_ADMINISTRATION',//configuration and others of sort that can come up
        PRE_ARRIVAL = 'PRE_ARRIVAL',
        POST_DEPARTURE = 'POST_DEPARTURE',
        ACTION_LIST = 'ACTION_LIST',
        INSIGHTS = 'INSTRUCTIONS',//legacy name for insights
        REPORTS = 'REPORTS',
        ANALYTICS = 'INSIGHTS',//this is now called analytics, but in backend the name of the feature remained the same!!
        TEMPLATES = 'TEMPLATES',
        DOWNLOAD_INSIGHTS = 'DOWNLOAD_INSIGHTS',
        INSIGHT_PUBLISHING = 'INSIGHT_PUBLISHING',
        PPT_REQUEST_ESTIMATE = 'PPT_REQUEST_ESTIMATE',
        VMS_UPLOAD = 'VMS_UPLOAD',
        SOF_TRANSCRIPTION = 'SOF_TRANSCRIPTION'
    }

    enum USER_ENTITLEMENTS {
        DRY = 'dry',
        WET = 'wet',
        INSIGHTS = 'INSIGHTS',
        ACTION_LIST = 'ACTION_LIST',
        TEMPLATES = 'TEMPLATES',
        ACCESS = 'access',
        REPORTS = 'REPORTS',
        SOF_TRANSCRIPTION = 'SOF_TRANSCRIPTION',
        ASSOCIATION = 'association',
        PTL_INSIGHTS_MANAGER = 'PTL_INSIGHTS_MANAGER',
        ELITE_PLUS_PLUS = 'ElitePlusPlus',
        POST_DEPARTURE = 'POST_DEPARTURE',
        PTL_OPTIMIZATION_TEAM = 'PTL_OPTIMIZATION_TEAM',
        INSIGHT_MANAGEMENT = 'INSIGHT_MANAGEMENT',
        PTL_OPTIMIZATION_MANAGER = 'PTL_OPTIMIZATION_MANAGER',
        VMS_UPLOAD = 'VMS_UPLOAD',
        PRE_ARRIVAL = 'PRE_ARRIVAL',
        DOWNLOAD_INSIGHTS = 'DOWNLOAD_INSIGHTS',
        PTL_OPERATOR = 'PTL_OPERATOR',
        INSTRUCTIONS = 'INSTRUCTIONS',
        PPT_REQUEST_ESTIMATE = 'PPT_REQUEST_ESTIMATE',
    }

    //Models

    interface Assignee {
        id?: number;
        name?: string;
        checked?: boolean;
        counter?: number;
    }

    interface AttachmentFileMetadata {
        progress?: number;
        uploaded?: boolean;
        downloadId?: string;
        failed?: boolean;
        valid?: boolean;
    }

    interface AttachmentFileType {
        extension?: string | string[],
        fileSignatureName?: FilesSignatureName,
        mimeType?: string,
        offset?: number,
        signature?: Uint8Array,
        signatureLength?: number
    }

    interface AttachmentFile {
        id?: UUID | number;
        name?: string;
        extension?: string;
        size?: number;
        file?: File;
        type?: AttachmentFileType,
        metadata?: AttachmentFileMetadata;
    }

    interface AttachmentFileResponse {
        addedAt?: string;
        addedBy?: string;
        filename?: string;
        id?: number;
    }

    interface Category {
        id?: number;
        name?: string;
        count: number;
    }

    interface Commodity {
        commodities?: Commodity[];
        depth?: number;
        id?: number;
        name?: string;
        parentId?: number;
        isSpecific?: boolean;
        type?: string;
        suggested?: boolean;
    }

    interface Coordinate {
        latitude?: number;
        longitude?: number;
    }

    interface Country {
        id?: number;
        name?: string;
        alreadyAdded?: boolean;
    }

    interface IApiResponse<T> {
        totalItems?: number;
        totalRecords?: number;
        items?: T[];
        counters?: { [counter: string]: number }
    }

    interface CountryResponse extends IApiResponse<Country> {
    }

    interface FileUploadConfig {
        config?: angular.IRequestConfig;
        onSuccess?: any;
        onError?: any;
    }

    interface InstructionCreatorEvent {
        type: InstructionCreatorEventType;
        payload: any;
    }

    interface InstructionCreatorConfig {
        fileUpload?: FileUploadConfig;
        richText?: quill.QuillOptionsStatic;
        username?: string;
    }

    export interface InstructionCreatorBreadCrumbs {
        previousPage?: string,

        // for PLO Pre and Post
        description?: {
            vesselName?: string,
            voyageName?: string,
            operation?: string,
            portName?: string,
            terminalName?: string,
            berthName?: string
        }
    }

    export interface Recipient {
        id: number,
        name: string
    }

    export interface OperationType {
        id: number,
        name: string,
        suggested: boolean
    }

    export interface IInstructionTags {
        country?: Country[];
        port?: Port[];
        terminal?: Terminal[];
        commodity?: marcura.Commodity[];
        specificCommodity?: marcura.Commodity[];
        vessels?: Vessel[];
        operationType?: OperationType;
        vesselType?: VesselType;
        counterparts?: ICounterpart[];
    }

    export interface IInstructionDTO extends IInstructionTags {
        id?: number;
        alertId?: number,
        alerts?: any[],
        text?: string;
        customTitle?: string;
        recipients?: Recipient[];
        assignees?: Assignee[];
        attachedIds?: number[];
        attachments?: AttachmentFile[];
        addedBy?: string;
        addedAt?: string;
        archivedBy?: string;
        archivedAt?: string;
        instructionCategory?: marcura.Category;
        instructionCategories?: marcura.Category[];
        important?: boolean;
        validFrom: string;
        validTo: string;
    }

    export interface InstructionCreatorDescription extends IInstructionDTO {
        breadCrumbs?: InstructionCreatorBreadCrumbs,
        publishedAt?: string,
        $canManage?: boolean
    }

    interface Operation {
        id?: number;
        name?: string;
        checked?: boolean;
    }

    interface Place {
        averageCost?: number;
        averageCostWithoutTowage?: number;
        averageCostWithTowage?: number;
        portcallsWithTowage?: number;
        portcallsWithoutTowage?: number;
        coordinate?: Coordinate;
        id?: string | number;
        maxCost?: number;
        minCost?: number;
        name?: string;
    }

    interface Port extends Place {
        aliases?: string[];
        country?: { id?: number, name?: string, code?: string };
        countryId?: number;
        locationCode?: string;
        rateOfExchange?: number;
        alreadyAdded?: boolean;
    }

    interface Tag {
        label?: string;
        isActive?: boolean;
        meta?: { [propertyName: string]: any };
    }

    interface Terminal extends Place {
        maxDraft?: number;
        port?: Port;
        totalTurnaround?: any;
        portId?: number;
    }

    export interface VesselType {
        id?: number;
        name?: string;
        dwtMin?: number;
        dwtMax?: number;
        dwt?: number;
        imo?: string;
    }


    interface UUID extends String {
    }

    interface Vessel {
        id?: number;
        imo?: string;
        name?: string;
        suggested: boolean;
        dwtMin?: number;
        dwtMax?: number;
    }

    interface IInstructionEvent {
        name?: string;
        alternative?: any;
        eventId?: number;
        eventTime?: any;
        lineRemark?: any;
        originalEventName?: string;
        rowNum?: any;
    }

    interface IInstructionCategory {
        id?: number;
        name?: string;
        truncatedName?: string;
        count?: number;
        selected?: boolean;
        truncated?: boolean;
        subCategories?: Array<IInstructionCategory>;
    }

    interface IInstructionAttachment {
        addedAt?: string;
        addedBy?: string;
        content?: any;
        filename?: string;
        id?: number;
        size?: number;
    }

    interface IInstruction {
        $addedAt?: string;
        $archivedAt?: string;
        $canManage: boolean;
        $canPublish: boolean;
        $hide?: boolean;
        $isOwner?: boolean;
        $publishedAt?: string;
        $validFrom?: string,
        $validTo?: string,
        activity?: IInstructionActivity;
        addedAt?: string;
        addedBy?: string;
        addedByFullname?: string;
        modifiedAt?: string;
        modifiedBy?: string;
        modifiedByFullname?: string;
        archivedAt?: string;
        archivedBy?: string;
        archivedByFullname?: string;
        attached?: boolean;
        isAttached?: boolean;
        attachedIds?: number[];
        attachments?: IInstructionAttachment[];
        counterparts?: ICounterpart[],
        country?: Country[];
        endEvent?: IInstructionEvent;
        id?: number,
        instructionCategories?: IInstructionCategory[];
        instructionCategory?: IInstructionCategory;
        isArchived: boolean;
        isDraft?: boolean;
        isExpiring?: boolean;
        locationTags?: LocationTag[];
        modifiable?: boolean;
        operationType?: any;
        port?: Port[];
        publishedAt?: string;
        publishedBy?: string;
        publishedByFullname?: string;
        recipients?: Recipient[];
        shared?: boolean;
        specificCommodity?: any[];
        startEvent?: IInstructionEvent;
        subscribed?: any[];
        terminal?: Terminal[];
        text?: string;
        validFrom?: string,
        validTo?: string,
        vesselPools?: any[];
        vessels?: Vessel[];
        vesselType?: any;
        displayAuthorAsLastEditor?: boolean;
    }

    interface LocationTag {
        id: string;
        name: string;
        country?: Country;
        port?: Port;
        terminal?: Terminal;
    }

    interface ICountedIds<T> {
        [id: number]: T;
    }

    interface ICountedNames<T> {
        [name: string]: T;
    }

    interface IUserCountedNames<T> {
        [userId: string]: {
            count: T,
            fullname: string
        };
    }

    interface IInstructionsCounters {
        draft: number,
        pag: number;
        archived: number;
        expiring: number;
        active: number;
        publish: number;
        hasAttachments: number;
    }

    interface IInstructionsManagerCounters {
        categories: marcura.IInstructionCategory[];
        counters: IInstructionsCounters,
        creators: IUserCountedNames<number>,
        assignees: ICountedNames<number>,
        counterparts: ICountedNames<number>
    }

    interface IInstructionActivity {
        activityId?: number,
        activityName?: string,
        instructions?: IInstruction[];
    }


    interface IInstructionResponse {
        activities?: IInstructionActivity[];
        genericInstructions?: IInstruction[];
    }

    interface IInstructionTagsFilter {
        portId?: number[];
        countryId?: number[];
        terminalId?: number[];
        vesselImos?: string[];
        vesselSegmentIds?: number[];
        operationId?: number;
        commodityId?: number[];
        specificCommodityId?: number[];
        cargoId?: number;
        counterparts?: number[];
    }

    interface IInstructionCategoryFilter {
        instructionCategory?: number[];
    }

    interface IIPortcallFilter {
        portcallId?: number;
        portId?: number;
        terminalId?: number;
    }

    //Do not change any of these names because they are used for filtering sofs besides instructions.
    //This is used also when opening PLO from PLC.
    interface IInstructionFilter extends IInstructionTagsFilter, IInstructionCategoryFilter {
        archivedBy?: string;
        archivedFrom?: number;
        archivedTo?: number;
        assignedTo?: string;
        createdBy?: string[];
        createdFrom?: string;
        createdTo?: string;
        dwtMax?: number;
        dwtMin?: number;
        //instructions do not support quantity. It is intended for sofs filtering which seems to use same filter object
        weightMax?: number;//this is actually quantity in activity (Discharge X MT of Coal)
        weightMin?: number;
        endEvent?: number;
        recipient?: number[];
        startEvent?: number;
        status?: string[];
        templateId?: number;
        searchMode?: string;
        fullText?: string;
        has_attachments?: boolean;
    }

    interface IInstructionFilterSorter {
        sortOrder?: 'asc' | 'desc';
        sortBy?: string;
    }

    interface IInstructionFilterPaging {
        pageSize?: number;
        pageNum?: number;
    }

    interface IInstructionFilterConfig {
        filter?: IInstructionFilter;
        paging?: IInstructionFilterPaging;
        portcallFilter?: IIPortcallFilter;
        sorter?: IInstructionFilterSorter;
    }

    interface IPaging {
        hasNextPage?: boolean;
        pageNumber?: number;
    }

    interface IInstructions {
        groups: IInstructionGroup[];
        counts: IInstructionsManagerCounters;
    }

    interface IInstructionGroup {
        category: Category;
        instructions?: {
            items: IInstruction[];
            totalRecords: number;
        }
    }

    interface IInstructionsListObject {
        groups: IInstructionGroup[];
        totalInsights: number;
    }

    interface ITag {
        id?: number;
        label: string;
        value?: number;
        checked?: boolean;
        disabled?: boolean;
        color?: TagsColor;
        isLast?: boolean;
    }

    interface ITagService {
        getColorClassName(tag: ITag): string;
    }

    interface IInstructionConfig {
        router?: { manager?: string; creator?: string };
    }

    interface IUser {
        [property: string]: any;
    }

    interface ILike {
        name: string;
        id: string;
    }

    interface ICounterpart {
        id?: number;
        name?: string;
    }

    interface IDraft {
        id?: number;
        draft?: number;
        dwt?: number;
        date?: string;
        loa?: number;
        reportedBy?: string;
    }

    interface IGPort {
        extraInfo: {source: string, extras: string[], lastUpdated: string};
        locations: object[];
    }

    interface ILocationRestriction {
        location: {
            countryId: number,
            countryName: string,
            portId: number,
            portName: string,
            terminalId: number,
            terminalName: string,
            berthId: number,
            berthName: string
        },
        numberOfComments: number,
        restrictions: IRestriction[]
    }

    interface IRestriction {
        categoryId: number,
        categoryName: string,
        restrictionId: number,
        versionId: number,
        value: string,
        agents?: IAgent[]
    }

    interface IAgent {
        agentId: number,
        agentName: string,
        lastUpdated: Date
    }

    interface IRestrictionsInfo {
        extraInfo: {
            extras: string[];
        },
        locationRestrictions: ILocationRestriction[];
    }

    interface ITides {
        lastUpdatedAt: string;
        localTime: string;
        numSofsAwaitedTide: number;
        forecast: ITideForecast[];
    }

    interface ITideForecast {
        date: string;
        sunrise: string;
        sunset: string;
        tidalRange: number;
        tides: ITideForecastItem[];
    }

    interface ITideForecastItem {
        tideType: string;
        tideTime: string;
        hight: number;
        isNextTide: boolean;
    }

    enum ILDRateShape {
        DOT = 'dot',
        LINE = 'line',
        DOTTED_LINE = 'dotted-line'
    }

    interface ILDRateCoord {
        x: number;
        y: number;
        offset?: {
            x?:number;
            y?:number;
        }
        center?: boolean;
    }

    interface ILDRateLine {
        [field: string]: number
        min?: number;
        max?: number;
        ownCoAvg?: number;
        ownCoActual?: number;
        avg?: number;
    }

    interface ILDRateLegend {
        shape: ILDRateShape;
        text: string;
    }

    interface ILDRateChart {
        [cls: string]: ILDRateLine;
        gross: ILDRateLine;
        net: ILDRateLine;
    }

    interface ILDRateShapes {
        [field: string]: ILDRateShape;
        avg?: ILDRateShape;
        ownCoAvg?: ILDRateShape;
        ownCoActual?: ILDRateShape;
    }

    interface ILDRateConfig {
        shapes?: {[field: string]: ILDRateShape};
        chart?: ILDRateChart;
        legend?: ILDRateLegend[];
    }

    interface IInsightsCounterService {
        insightsCount: number;
        updateInsightsCount(total: number): void;
    }

    
    interface AnalyticsEmitData {
        event: {
            type?: string;
            url?: string;
            app?: string;
        };
        data?: any;
    }
    interface IEmitAnalytics {
        emitAnalyticsEvent(data: AnalyticsEmitData): void;
        emitDefaultClickAnalyticsEvent(category: string): void;
        emitDefaultOnChangeAnalyticsEvent(category: string): void;
        emitEditorTagsOnChangeAnalyticsEvent(label: string): void;
    }
}
